#!/bin/sh
############################################################################
#                                                                          #
#  Copyright (c) 2001, 2006 NoMachine, http://www.nomachine.com.           #
#                                                                          #
#  NXSERVER, NX protocol compression and NX extensions to this software    #
#  are copyright of NoMachine. Redistribution and use of the present       #
#  software is allowed according to terms specified in the file LICENSE    #
#  which comes in the source distribution.                                 #
#                                                                          #
#  Check http://www.nomachine.com/licensing.html for applicability.        #
#                                                                          #
#  NX and NoMachine are trademarks of Medialogic S.p.A.                    #
#                                                                          #
#  All rigths reserved.                                                    #
#                                                                          #
############################################################################
PATH="/usr/bin:/bin:/usr/local/bin" && export PATH

CONFIG_FILE="/usr/NX/etc/server.cfg"

COMMAND_ECHO="echo"
COMMAND_STAT="stat"
COMMAND_FLOCK="flock"
COMMAND_SED="sed"

if [ "x$1" != "xnx" -a "x$1" != "xroot" ];
then
  $COMMAND_ECHO "NX> 599 Error: nxconfigure.sh: only 'nx' or 'root' user can change config file by this script" 1>&2
  exit 1
fi

if test $# -ne 4 ; then
  $COMMAND_ECHO "NX> 599 Error: Wrong parameters count." 
  exit 1
fi

if [ -x /bin/grep ];
then 
  GREPCOMMAND="/bin/grep"
else
  GREPCOMMAND="grep"
fi

if [ -x /bin/rm ];
then
  RMCOMMAND="/bin/rm"
else 
  RMCOMMAND="rm"
fi

if [ -x /bin/cp ];
then
  CPCOMMAND="/bin/cp"
else
  CPCOMMAND="cp"
fi

if [ -x /bin/mv ];
then
  MVCOMMAND="/bin/mv"
else
  MVCOMMAND="mv"
fi

if [ ! -r $CONFIG_FILE ];
then
  $COMMAND_ECHO "Error: Unable to read file $CONFIG_FILE" 1>&2
  exit 1
fi

if [ ! -f $CONFIG_FILE.backup ];
then
  command="$CPCOMMAND -f $CONFIG_FILE $CONFIG_FILE.backup"
  result=`/bin/bash -c "$command" 2>&1`
else
  command="$RMCOMMAND -f $CONFIG_FILE.backup"
  result=`/bin/bash -c "$command" 2>&1`
  exitcode="$?"

  if [ $exitcode != 0 ] || [ x$result != x ];
  then
    $COMMAND_ECHO "NX> 500 ERROR: Problems while removing: $CONFIG_FILE.backup file: $result" 1>&2
    exit $exitcode
  else
    command="$CPCOMMAND -f $CONFIG_FILE $CONFIG_FILE.backup"
    result=`/bin/bash -c "$command" 2>&1`
  fi
fi
exitcode="$?"

if [ $exitcode != 0 ] || [ x$result != x ];
then
  $COMMAND_ECHO "NX> 500 ERROR: Problems while coping: $CONFIG_FILE to $CONFIG_FILE.backup: $result" 1>&2
  exit $exitcode
fi
  
EXPRESSION="^ *\#* *$3 *=.*"
command="$GREPCOMMAND '$EXPRESSION' $CONFIG_FILE "
result=`/bin/bash -c "$command" 2>&1`
GREP_RESULT="$?"
exitcode="0"
 
if [ "x$GREP_RESULT" = "x0" ];
then
  #key found, to be updated
  EXPRESSION="s|^ *\#* *$3 *=.*|$3 = \"$4\"|g"
  command="$COMMAND_FLOCK $CONFIG_FILE $COMMAND_SED -r '$EXPRESSION' $CONFIG_FILE > $CONFIG_FILE.tmp"
  result=`/bin/bash -c "$command" 2>&1`
  exitcode="$?"

  if [ $exitcode != 0 ] || [ x$result != x ];
  then
    $COMMAND_ECHO "NX> 500 ERROR: Problems while updating: $CONFIG_FILE.tmp file: $result" 1>&2
    exit $exitcode
  fi

  if [ -f $CONFIG_FILE.tmp ];
  then
    chmod 0644 $CONFIG_FILE.tmp
    chown root:root $CONFIG_FILE.tmp
    $COMMAND_ECHO "NX> 900 Moving $CONFIG_FILE.tmp in $CONFIG_FILE"
    command="$MVCOMMAND -f $CONFIG_FILE.tmp $CONFIG_FILE"
    result=`/bin/bash -c "$command" 2>&1`
    exitcode="$?"
  fi

  if [ $exitcode != 0 ] || [ x$result != x ];
  then
    $COMMAND_ECHO "NX> 500 ERROR: Problems while moving: $CONFIG_FILE.tmp file: $result" 1>&2
    exit $exitcode
  fi

  if [ -f $CONFIG_FILE.backup ];
  then
    $COMMAND_ECHO "NX> 900 Removing $CONFIG_FILE.backup"
    command="$RMCOMMAND -f $CONFIG_FILE.backup"
    result=`/bin/bash -c "$command" 2>&1`
    exitcode="$?"
  fi

  if [ $exitcode != 0 ] || [ x$result != x ];
  then
    $COMMAND_ECHO "NX> 500 ERROR: Problems while removing: $CONFIG_FILE.backup file: $result" 1>&2
    exit $exitcode
  else
    FINAL_RESULT="substituted"
  fi

elif [ "x$GREP_RESULT" = "x1" ];
then
  $COMMAND_ECHO "NX> 900 Key not found, appending the key to the end of file."
  KEY="$3 = \"$4\""
  command="$COMMAND_FLOCK $CONFIG_FILE $COMMAND_ECHO '$KEY' >> $CONFIG_FILE"
  result=`/bin/bash -c "$command" 2>&1`
  exitcode="$?"

  if [ $exitcode != 0 ] || [ x$result != x ];
  then
    $COMMAND_ECHO "NX> 500 ERROR: Problems while updating: $CONFIG_FILE file: $result" 1>&2
    exit $exitcode
  else
    FINAL_RESULT="appended"
  fi
fi

$COMMAND_ECHO "NX> 900 key $3 has been $FINAL_RESULT"

exit 0
